# %{ver}, %{rel} are provided by the Makefile 
%define ver @VERSION@
%define rel @RELEASE@

# CUPS version we are based on
%define cupsver @CUPSVER@

# OPVP Null driver version
%define opvpver @OPVPVER@

# hplip/hpijs version
%define hplipver @HPLIPVER@

# some defined dirs
%define troot /opt/lsb/test/printing
%define croot %{troot}/cups

Summary: LSB Runtime Printing Tests
Name: lsb-test-printing
Version: %{ver}
Release: %{rel}
License: Various
Group: Development/Tools
Source0: lsb-test-printing-%{version}.tar.gz
Source1: cups-%{cupsver}-source.tar.bz2
Source2: opvpnull-%{opvpver}.tar.gz
Source3: hplip-%{hplipver}.tar.gz
Patch0: cups-1.1.23-lsb-no-getpass.patch
Patch1: hplip-2.8.12-lsbcc.patch
Patch2: lsb-test-printing-add-ipp-tests.patch
Patch3: lsb-test-printing-fix-cups-build-50.patch
URL: http://www.linuxbase.org/test
BuildRoot: %{_tmppath}/%{name}-root
AutoReqProv: no
Requires: lsb-tet3-lite
BuildRequires: lsb-tet3-lite-devel

%description
This package provides the LSB Runtime Printing Tests, 
for distribution certification.

%prep
%setup -q -n printing-test -a1 -a2 -a3
cd cups-%{cupsver}
%patch0 -p0 -b .lsbcc
cd ..
cd hplip-%{hplipver}
%patch1 -p0 -b .lsbcc
cd ..
%patch2 -p0 -b .bug-3087
%patch3 -p0 -b .lsb-5.0

%build
export LSBCC_BESTEFFORT=1
# first CUPS
cd cups-%{cupsver}
CC=/opt/lsb/bin/lsbcc ./configure \
	--prefix=/opt/lsb/test/printing/cups \
	--disable-slp \
	--disable-ssl \
	--disable-openssl \
	--disable-gnutls \
	--disable-cdsassl

# just make the bits we need
make -C cups
sed -i 's|-lcups|../cups/libcups.a|g' Makedefs
make -C scheduler cupsd

# a couple of filters
cd filter
make pstops texttops
cd ..

# some command line tools
cd systemv
make lpadmin lpstat
cd ..
cd berkeley
make lpr
make lprm
cd ..

cd ..

# opvpnull driver
cd opvpnull
CC=/opt/lsb/bin/lsbcc ./autogen.sh --prefix=/opt/lsb
make
cd ..

# hpijs
cd hplip-%{hplipver}
CC=lsbcc CXX=lsbc++ ./configure --enable-hpijs-only-build
make
cd ..

# lsb_issymlink for the fhs test
cd BIN
lsbcc -o lsb_issymlink lsb_issymlink.c
cd ..
 
# now the tests
make
 
%install
install -d ${RPM_BUILD_ROOT}%{troot}

# build up our private cups install
for dir in ppd certs interfaces;do
  install -d ${RPM_BUILD_ROOT}%{croot}/etc/cups/$dir
done

# seems modern systems complain about raw printing
sed -i 's|^#application/octet-stream|application/octet-stream|g' cups-%{cupsver}/conf/mime.convs
sed -i 's|^#application/octet-stream|application/octet-stream|g' cups-%{cupsver}/conf/mime.types

# some minimal configuration files
install ppd/test.ppd ${RPM_BUILD_ROOT}%{croot}/etc/cups/ppd/LSBTest.ppd
touch ${RPM_BUILD_ROOT}%{croot}/etc/cups/classes.conf
install cups-%{cupsver}/conf/mime.convs ${RPM_BUILD_ROOT}%{croot}/etc/cups
install cups-%{cupsver}/conf/mime.types ${RPM_BUILD_ROOT}%{croot}/etc/cups

cat > ${RPM_BUILD_ROOT}%{croot}/etc/cups/lpoptions << EOF
default LSBTest page-top=36 page-bottom=36 page-left=36 page-right=36 cpi=12 lpi=7 wrap=true scaling=100
EOF

cat > ${RPM_BUILD_ROOT}%{croot}/etc/cups/printers.conf.dist << EOF
<DefaultPrinter LSBTest>
Info Fake Printer for LSB testing
Location Office
DeviceURI file://dev/null
State Stopped
StateMessage 
Accepting Yes
JobSheets none none
QuotaPeriod 0
PageLimit 0
KLimit 0
</Printer>
EOF

cat > ${RPM_BUILD_ROOT}%{croot}/etc/cups/cupsd.conf << EOF
LogLevel info
Port 632
TempDir /opt/lsb/test/printing/cups/var/spool/cups/tmp

BrowseAddress @LOCAL
BrowseDeny All
BrowseAllow @LOCAL
BrowseOrder deny,allow

<Location />
Order Deny,Allow
Deny From All
Allow From All
</Location>         

<Location /admin>
AuthType None
AuthClass Anonymous
Order Deny,Allow
Deny From All
Allow From @LOCAL
</Location>

BrowseInterval 0
Browsing Off
FileDevice yes
EOF

# (sb) alternate config file for systems unable to use alternate port (bug 2483)
sed 's/632/631/' ${RPM_BUILD_ROOT}%{croot}/etc/cups/cupsd.conf > ${RPM_BUILD_ROOT}%{croot}/etc/cups/cupsd.conf.631

for dir in model banners;do
  install -d ${RPM_BUILD_ROOT}%{croot}/share/cups/$dir
done
for dir in log spool;do
  install -d ${RPM_BUILD_ROOT}%{croot}/var/$dir/cups
done
for dir in backend filter;do
  install -d ${RPM_BUILD_ROOT}%{croot}/lib/cups/$dir
done

install -d ${RPM_BUILD_ROOT}%{croot}/client
cat > ${RPM_BUILD_ROOT}%{croot}/client/.cupsrc << EOF
ServerName localhost:632
EOF

# seems newer systems use .cups/client.conf
install -d ${RPM_BUILD_ROOT}%{croot}/client/.cups
cd ${RPM_BUILD_ROOT}%{croot}/client/.cups
ln -sf ../.cupsrc client.conf
cd -

# need these 2 filters
install cups-%{cupsver}/filter/pstops ${RPM_BUILD_ROOT}%{croot}/lib/cups/filter
install cups-%{cupsver}/filter/texttops ${RPM_BUILD_ROOT}%{croot}/lib/cups/filter

# these command line tools
install -d ${RPM_BUILD_ROOT}%{croot}/bin
install cups-%{cupsver}/berkeley/lpr ${RPM_BUILD_ROOT}%{croot}/bin
install cups-%{cupsver}/berkeley/lprm ${RPM_BUILD_ROOT}%{croot}/bin
install cups-%{cupsver}/systemv/lpstat ${RPM_BUILD_ROOT}%{croot}/bin
install -d ${RPM_BUILD_ROOT}%{croot}/sbin
install cups-%{cupsver}/systemv/lpadmin ${RPM_BUILD_ROOT}%{croot}/sbin

# and the daemon
install cups-%{cupsver}/scheduler/cupsd ${RPM_BUILD_ROOT}%{croot}/sbin

# opvpnull driver
install -d ${RPM_BUILD_ROOT}%{troot}/drivers
install opvpnull/.libs/libopvpnull.so ${RPM_BUILD_ROOT}%{troot}/drivers
install opvpnull/.libs/libopvpnull.so.0.0.1 ${RPM_BUILD_ROOT}%{troot}/drivers

# hpijs
install hplip-%{hplipver}/hpijs ${RPM_BUILD_ROOT}%{troot}/drivers

# finally the tests

# tweak tetexec.cfg for the version/release/arch
%ifarch %{ix86}
%define tarch ia32
%else
%define tarch %{_arch}
%endif
echo "VSX_NAME=%{name} %{version}-%{rel} (%{tarch})" >> tetexec.cfg

# and install
for dir in convenience fhs ipp ppd raster testgs testfoomaticrip results BIN LIB;do
  install -d ${RPM_BUILD_ROOT}%{troot}/$dir
done
install convenience/cupsConvenience ${RPM_BUILD_ROOT}%{troot}/convenience
install cups-%{cupsver}/cups/testipp ${RPM_BUILD_ROOT}%{troot}/ipp
install ipp/LSBTest.ipp ${RPM_BUILD_ROOT}%{troot}/ipp
install ppd/cupsPPD ${RPM_BUILD_ROOT}%{troot}/ppd
install raster/cupsRaster ${RPM_BUILD_ROOT}%{troot}/raster

install testgs/testgs.sh ${RPM_BUILD_ROOT}%{troot}/testgs/testgs
install testgs/testijs.sh ${RPM_BUILD_ROOT}%{troot}/testgs/testijs

install testfoomaticrip/testfoomaticrip.sh ${RPM_BUILD_ROOT}%{troot}/testfoomaticrip/testfoomaticrip
install testfoomaticrip/foomatic-test* ${RPM_BUILD_ROOT}%{troot}/testfoomaticrip

install -d ${RPM_BUILD_ROOT}%{troot}/fhs/share-ppd
install fhs/share-ppd/share-ppd-tc.sh ${RPM_BUILD_ROOT}%{troot}/fhs/share-ppd/share-ppd

install tet_scen ${RPM_BUILD_ROOT}%{troot}
install tetexec.cfg ${RPM_BUILD_ROOT}%{troot}
install run_tests ${RPM_BUILD_ROOT}%{troot}
install LIB/*.sh ${RPM_BUILD_ROOT}%{troot}/LIB
install BIN/lsb_issymlink ${RPM_BUILD_ROOT}%{troot}/BIN

install ppd/test.ppd ${RPM_BUILD_ROOT}%{troot}/ppd
touch ${RPM_BUILD_ROOT}%{troot}/ppd/notest.ppd
install ppd/test.ppd ${RPM_BUILD_ROOT}%{troot}/convenience
install convenience/testprint.ps ${RPM_BUILD_ROOT}%{troot}/convenience
install raster/test.raster ${RPM_BUILD_ROOT}%{troot}/raster

install convenience/testprint.ps ${RPM_BUILD_ROOT}%{troot}/testgs
install opvpnull/test/example.ps ${RPM_BUILD_ROOT}%{troot}/testgs
install testgs/tiger.ps ${RPM_BUILD_ROOT}%{troot}/testgs

# dummy config file for cupsPutFd, cupsPutFile
cat > ${RPM_BUILD_ROOT}%{troot}/convenience/lsb.conf << EOF
# LSB dummy config file for cupsPutFd, cupsPutFile
EOF

# foomatic-rip note for ubuntu
cat > ${RPM_BUILD_ROOT}%{troot}/README.testfoomaticrip << EOF
Note that on Ubuntu Gutsy and later you must run

sudo aa-complain cupsd

to be able to run the tests. After the tests

sudo aa-enforce cupsd
EOF

# license files
install -d ${RPM_BUILD_ROOT}/opt/lsb/doc/%{name}
cp cups-%{cupsver}/LICENSE.txt ${RPM_BUILD_ROOT}/opt/lsb/doc/%{name}/LICENSE.cups
cp opvpnull/LICENSE.txt ${RPM_BUILD_ROOT}/opt/lsb/doc/%{name}/LICENSE.opvpnull
cp Artistic ${RPM_BUILD_ROOT}/opt/lsb/doc/%{name}
cat > ${RPM_BUILD_ROOT}/opt/lsb/doc/%{name}/README.Licenses << EOF
Test code produced by The Linux Foundation and contributors is licensed
under the Artistic license. CUPS and the opvpnull driver have their own
licenses, included in this directory.
EOF

%clean
rm -rf $RPM_BUILD_ROOT

%files
%defattr(-,root,root)
%dir /opt/lsb/test/printing
/opt/lsb/test/printing
%dir /opt/lsb/doc/%{name}
/opt/lsb/doc/%{name}/*

%changelog
* Tue Feb 05 2013 Stew Benedict <stewb@linux-foundation.org>
- fix the cupsd build for LSB-5.0 header changes (need sys/select.h, P3)

* Mon May 14 2012 Stew Benedict <stewb@linux-foundation.org>
- adapt cups tests for ippReadIO, ippWriteIO, ippReadFile (bug 3087, P2)
 
* Fri Jun 03 2011 Stew Benedict <stewb@linux-foundation.org>
- newer systems use .cups/client.conf instead of .cupsrc to get a 
  custom port, add a symlink for this (bug 3256)

* Mon Apr 19 2010 Stew Benedict <stewb@linux-foundation.org>
- ipp/http interfaces in LSB now, drop static archive build, 
  re-enable tests we disabled in 3.2

* Wed Feb 03 2010 Stew Benedict <stewb@linux-foundation.org>
- build/package our own lprm too

* Mon Mar 02 2009 Stew Benedict <stewb@linux-foundation.org>
- add our own hpijs command for the gs tests (S3, P1, bug 2553)

* Fri Jan 16 2009 Stew Benedict <stewb@linux-foundation.org>
- rework run_tests to deal with cups-1.1.x systems that
  cannot use alternate port for client apps (bug 2483)

* Fri Nov 14 2008 Stew Benedict <stewb@linux-foundation.org>
- add test for /usr/share/ppd (bug 2157)

* Tue Sep 02 2008 Stew Benedict <stewb@linux-foundation.org>
- rediff P0 (bug 2273)

* Thu Jan 10 2008 Stew Benedict <stewb@linux-foundation.org>
- replace 127.0.0.1 in cupsd.conf with @LOCAL (cupsGetPPD failing in places)

* Mon Dec 03 2007 Stew Benedict <stewb@linux-foundation.org>
- add license files

* Tue Nov 27 2007 Stew Benedict <stewb@linux-foundation.org>
- add P0 from Till (getpass workaround for cups)

* Thu Nov 22 2007 Till Kamppeter <till.kamppeter@gmail.com>
- Made testfoomaticrip support private copy of CUPS
- Added command line tools lpadmin, lpstat, and lpr to private copy of CUPS
  (needed by testfoomaticrip)
- Added test for foomatic-rip being available as a CUPS filter to
  testfoomaticrip
- Added tests for page overrides to foomatic-rip
- Updated number of tests for testfoomaticrip in tet_scen

* Tue Nov 20 2007 Stew Benedict <stewb@linux-foundation.org>
- testfoomaticrip from Till Kamppeter

* Thu Nov 15 2007 Stew Benedict <stewb@linux-foundation.org>
- install tiger.ps (pxlmono, pxlcolor tests from George Liu)

* Mon Nov 12 2007 Stew Benedict <stewb@linux-foundation.org>
- add opvpnull driver, modify testgs to use it

* Tue Oct 09 2007 Stew Benedict <stewb@linux-foundation.org>
- update testgs to process a known file with each of the drivers

* Tue Oct 02 2007 Stew Benedict <stewb@linux-foundation.org>
- Initial package


