/* cupsConvenience.c test CUPS Convenience ABI interfaces */
/* 9/26/2007 Stew Benedict stewb@linux-foundation.org */

/* interfaces:
cupsAddDest
cupsAddOption
cupsCancelJob
cupsEncryption
cupsFreeDests
cupsFreeJobs
cupsFreeOptions
cupsGetDefault
cupsGetDefault2
cupsGetDest
cupsGetDests
cupsGetDests2
cupsGetFd
cupsGetFile
cupsGetJobs
cupsGetJobs2
cupsGetOption
cupsGetPPD
cupsGetPPD2
cupsGetPassword
cupsLangEncoding
cupsLangDefault
cupsLangFlush
cupsLangFree
cupsLangGet
cupsLastError
cupsMarkOptions
cupsParseOptions
cupsPrintFile
cupsPrintFile2
cupsPrintFiles
cupsPrintFiles2
cupsPutFd
cupsPutFile
cupsServer
cupsSetDests
cupsSetDests2
cupsSetEncryption
cupsSetPasswordCB
cupsSetServer
cupsSetUser
cupsTempFd
cupsUser
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <tet_api.h>
#include <cups/ipp.h>
#include <cups/http.h>
#include <cups/cups.h>
#include <cups/ppd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

/* some actions reload the server, defined delay */
#define WAIT_RELOAD 10

const char testpw[15] = "cupstestpasswd";
const char testhost[30] = "cupshost.linux-foundation.org";
const char testuser[9] = "cupsuser";
const char getfile[17] = "/ppd/LSBTest.ppd";
const char getppd[8] = "LSBTest";
const char putfile[21] = "/admin/conf/lsb.conf";
const char config[9] = "lsb.conf";
const char printer[8] = "LSBTest";
const char pfile[13] = "testprint.ps";
const char prompt[25] = "Please enter a password:";
const char ppdfile[9] = "test.ppd";
const char oname[6] = "media";
const char ovalue[7] = "letter";
const char jtitle[10] = "Job Title";
const char dname[4] = "foo";
const char instance[4] = "bar";

void test_cleanup();
void (*tet_startup)() = NULL;
void (*tet_cleanup)() = test_cleanup;
void test1();
void test2();
void test3();
void test4();
void test5();
void test6();
void test7();
void test8();
void test9();
void test10();
void test11();
void test12();
void test13();
void test14();
void test15();
void test16();
void test17();
void test18();
void test19();
void test20();
void test21();
void test22();
void test23();
void test24();
void test25();
void test26();
void test27();
void test28();
void test29();
void test30();
void test31();
void test32();
void test33();
void test34();
void test35();
void test36();
void test37();
void test38();
void test39();
void test40();
void test41();
void test42();
void test43();

struct tet_testlist tet_testlist[] = 
{ 
    {test1,1},
    {test2,2},
    {test3,3},
    {test4,4},
    {test5,5},
    {test6,6},
    {test7,7},
    {test8,8},
    {test9,9},
    {test10,10},
    {test11,11},
    {test12,12},
    {test13,13},
    {test14,14},
    {test15,15},
    {test16,16},
    {test17,17},
    {test18,18},
    {test19,19},
    {test20,20},
    {test21,21},
    {test22,22},
    {test23,23},
    {test24,24},
    {test25,25},
    {test26,26},
    {test27,27},
    {test28,28},
    {test29,29},
    {test30,30},
    {test31,31},
    {test32,32},
    {test33,33},
    {test34,34},
    {test35,35},
    {test36,36},
    {test37,37},
    {test38,38},
    {test39,39},
    {test40,40},
    {test41,41},
    {test42,42},
    {test43,43},
    {NULL,0} 
};

void test_cleanup()
{

}

/* for test23,39 */ 
const char * test_password_cb(const char *prompt)
{
    return(testpw);
}

void test1()
{
    const char      *defprinter;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetDefault is called to get the default printer or class for the default server, the function returns the default printer or NULL");

    /* FIXME - returns NULL with no cupsd setup */
    if((defprinter = cupsGetDefault()) == NULL) {
        tet_printf("cupsGetDefault returned NULL, expected a value");
        tet_result(TET_FAIL);
    } else {
        tet_printf("cupsGetDefault returned %s", defprinter);
        tet_result(TET_PASS);
    }    
}

void test2()
{
    const char      *defprinter;
    http_t          *http;
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetDefault2 is called to get the default printer or class for the default server, the function returns the default printer or NULL");

    http = httpConnect(cupsServer(), ippPort());
    if((defprinter = cupsGetDefault2(http)) == NULL) {
        tet_printf("cupsGetDefault2 returned NULL, expected a value");
        tet_result(TET_FAIL);
    } else {
        tet_printf("cupsGetDefault2 returned %s", defprinter);
        tet_result(TET_PASS);
    }
    httpClose(http);
}

void test3()
{
    const char          *ppd;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetPPD is called to get a PPD file for a printer on the default server, the function returns the filename of the PPD file");

    if((ppd = cupsGetPPD(getppd)) == NULL){
        tet_printf("cupsGetPPD returned NULL, expected a filename");
        tet_result(TET_FAIL);
    } else {
        tet_printf("cupsGetPPD returned %s", ppd);
        tet_result(TET_PASS);
    }
}

void test4()
{
    const char          *ppd;
    http_t              *http;
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetPPD2 is called to get a PPD file for a printer on the default server, the function returns the filename of the PPD file");

    http = httpConnect(cupsServer(), ippPort());
    if((ppd = cupsGetPPD2(http, getppd)) == NULL){
        tet_printf("cupsGetPPD2 returned NULL, expected a filename");
        tet_result(TET_FAIL);
    } else {
        tet_printf("cupsGetPPD2 returned %s", ppd);
        tet_result(TET_PASS);
    }
    httpClose(http);
}

void test5()
{
    int                 num_options = 0;
    cups_option_t       *options;
    int                 job_id;
    int                 result;

    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsPrintFile is called to print a file to a printer or class on the default server, the function returns the Job ID");

    job_id = cupsPrintFile(printer, pfile, jtitle, num_options, options);
    if (job_id < 1) {
        tet_printf("cupsPrintFile returned %d, expected a positive value", job_id);
        tet_result(TET_FAIL);
    } else {
        tet_printf("Job ID: %d", job_id);
        tet_result(TET_PASS);
        result = cupsCancelJob(printer, job_id);
    }
}

void test6()
{
    int                 num_options = 0;
    int                 job_id;
    http_t              *http;
    int                 result;
    cups_option_t       *options;

    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsPrintFile2 is called to print a file to a printer or class on the default server, the function returns the Job ID");

    http = httpConnect(cupsServer(), ippPort());
    job_id = cupsPrintFile2(http, printer, pfile, jtitle, num_options, options);
    if (job_id == 0) {
        tet_printf("cupsPrintFile2 returned 0, expected a value");
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
        result = cupsCancelJob(printer, job_id);
    }
    httpClose(http);
}

void test7()
{
    int                 num_options = 0;
    cups_option_t       *options;
    int                 job_id;
    const char          *files[2];
    int                 num_files = 2;
    int                 result;

    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsPrintFiles is called to print one or more files to a printer or class on the default server, the function returns the Job ID");
    
    /* just 2 of the same file to test */
    files[0] = pfile;
    files[1] = pfile;

    job_id = cupsPrintFiles(printer, num_files, files, jtitle, num_options, options);
    if (job_id < 1) {
        tet_printf("cupsPrintFiles returned %d, expected a positive value", job_id);
        tet_result(TET_FAIL);
    } else {
        tet_printf("cupsPrintFiles returned %d", job_id);
        tet_result(TET_PASS);
        result = cupsCancelJob(printer, job_id);
    }
}

void test8()
{
    int                 num_jobs;
    cups_job_t          *jobs;
    int                 num_options = 0;
    cups_option_t       *options;
    int                 job_id;
    int                 result;

    options = (cups_option_t *)0;
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetJobs is called to get the jobs from the default server, the function returns the number of jobs");

    /* put a job in the queue */
    job_id = cupsPrintFile(printer, pfile, jtitle, num_options, options);
    if (job_id < 1) {
        tet_printf("cupsPrintFile returned %d, expected a positive value", job_id);
        tet_result(TET_UNRESOLVED);
        return;
    } else {
        tet_printf("Job ID: %d", job_id);
    }

    /* XXX - FIXME - need to fake a stalled job to get an answer */
    if((num_jobs = cupsGetJobs(&jobs, NULL, 0, 0)) == 0) {
        tet_printf("cupsGetJobs returned 0, expecting a number of jobs");
        tet_result(TET_FAIL);
    } else {
        tet_printf("cupsGetJobs returned %d", num_jobs);
        tet_result(TET_PASS);
    }
    result = cupsCancelJob(printer, job_id);
    cupsFreeJobs(num_jobs, jobs);
}

void test9()
{
    int                 num_jobs;
    cups_job_t          *jobs;
    http_t              *http;
    int                 result;
    int                 num_options = 0;
    cups_option_t       *options;
    int                 job_id;

    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetJobs2 is called to get the jobs from the default server, the function returns the number of jobs");

    /* put a job in the queue */
    job_id = cupsPrintFile(printer, pfile, jtitle, num_options, options);
    if (job_id < 1) {
        tet_printf("cupsPrintFile returned %d, expected a positive value", job_id);
        tet_result(TET_UNRESOLVED);
        return;
    } else {
        tet_printf("Job ID: %d", job_id);
    }
    
    /* XXX - FIXME - need to fake a stalled job to get an answer */
    http = httpConnect(cupsServer(), ippPort());
    if((num_jobs = cupsGetJobs2(http, &jobs, NULL, 0, 0)) == 0) {
        tet_printf("cupsGetJobs returned 0, expecting a number of jobs");
        tet_result(TET_FAIL);
    } else {
        tet_printf("cupsGetJobs returned %d", num_jobs);
        tet_result(TET_PASS);
    }
    result = cupsCancelJob(printer, job_id);
    cupsFreeJobs(num_jobs, jobs);
    httpClose(http);
}

void test10()
{
    int             org_dests = 0;
    int             num_dests = 0;
    cups_dest_t     *dests;

    dests = (cups_dest_t *)0;

    /* free any existing dests - also part of bug 2645 issues, value returned would vary */
    cupsFreeDests(num_dests, dests);

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsAddDest is called to add a destination to the list of destinations, the function returns the new number of destinations");

    /* see bug 2645 - need to call cupsGetDests first */
    org_dests = cupsGetDests(&dests);

    /* see bug 3440 - don't always get org_dests + 1 either? Just accept >= 0 # */
    if((num_dests = cupsAddDest(dname, NULL, 1, &dests)) < 0) {
        tet_printf("cupsAddDest returned %d, expected >= 0", num_dests);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }

    cupsFreeDests(num_dests, dests);
}

void test11()
{
    int             num_options = 0;
    cups_option_t   *options;

    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsAddOption is called to add an option to the options array, the function returns the number of options");

    if((num_options = cupsAddOption(oname, ovalue, 0, &options)) != 1) {
        tet_printf("cupsAddOption returned %d, expected 1", num_options);
        tet_result(TET_FAIL);

    } else {
        tet_result(TET_PASS);
    }

    /* valgrind finding a number of issues in these tests, try to clean things up */
    cupsFreeOptions(num_options, options);
}

void test12()
{
    int                 num_options = 0;
    cups_option_t       *options;
    int                 job_id;
    int                 result;

    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsCancelJob is called to cancel a job on the default print server, the function returns 1 on success, 0 on failure");

    /* put a job in the queue */
    job_id = cupsPrintFile(printer, pfile, jtitle, num_options, options);
    if (job_id < 1) {
        tet_printf("cupsPrintFile returned %d, expected a positive value", job_id);
        tet_result(TET_UNRESOLVED);
        return;
    } else {
        tet_printf("Job ID: %d", job_id);
    }
   
    /* XXX - FIXME - with no cups running, returns 0 */
    if((result = cupsCancelJob(printer, job_id)) == 0) {
        tet_printf("cupsCancelJob returned 0, expected 1");
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }

    cupsFreeOptions(num_options,options);
}

void test13()
{
    http_encryption_t       encrypt;
    http_encryption_t       encrypt_org;
    http_encryption_t       encrypt_set = HTTP_ENCRYPT_REQUIRED;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsEncryption is called to get the default encryption settings, the function returns the encryption settings");

    /* get the current setting */
    encrypt_org = cupsEncryption();

    /* set a value, then read it */
    errno = 0;
    cupsSetEncryption(encrypt_set);
    if(errno) {
        tet_printf("cupsSetEncryption returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_UNRESOLVED);
        return;
    }

    if((encrypt = cupsEncryption()) != encrypt_set) {
        tet_printf("cupsEncryption returned %d, expected %d", encrypt, encrypt_set);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    cupsSetEncryption(encrypt_org);
}

void test14()
{
    int             org_dests = 0;				
    int             num_dests = 0;
    cups_dest_t     *dests;
    cups_dest_t     *dest;

    /* crashes with SIGSEGV */
    tet_result(TET_UNTESTED);
    return;

    dest = (cups_dest_t *)0;
    dests = (cups_dest_t *)0;

    /* free any existing dests - also part of bug 2645 issues, value returned would vary */
    cupsFreeDests(num_dests, dests);

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsFreeDests is called to free the memory used by the list of destinations, the function returns nothing");

    /* see bug 2645 - need to call cupsGetDests first */
    org_dests = cupsGetDests(&dests);

    /* setup a fake destination */
    if((num_dests = cupsAddDest(dname, NULL, 1, &dests)) != org_dests + 1) {
        tet_printf("cupsAddDest returned %d, expected %d", num_dests, org_dests + 1);
        tet_result(TET_UNRESOLVED);
        cupsFreeDests(num_dests, dests);
        return;
    }

    num_dests = cupsGetDests(&dests);
    dest      = cupsGetDest(NULL, NULL, 1, dests);

    errno = 0;    
    cupsFreeDests(num_dests, dests);
    if(errno) {
        tet_printf("cupsFreeDests returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }

}

void test15()
{
    int             num_jobs;
    cups_job_t     *jobs;

    /* tcc kills this with SIGTERM */
    tet_result(TET_UNTESTED);
    return;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsFreeJobs is called to free memory used by job data, the function returns nothing");

    num_jobs = cupsGetJobs(&jobs, NULL, 0, 0);
    
    errno = 0;
    cupsFreeJobs(num_jobs, jobs);
    if(errno) {
        tet_printf("cupsFreeJobs returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }    
}

void test16()
{
    int             num_options = 0;
    cups_option_t   *options;

    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsFreeOptions is called to free memory used by options, the function returns nothing");

    /* setup an option */
    if((num_options = cupsAddOption(oname, ovalue, num_options, &options)) != 1) {
        tet_printf("cupsAddOption returned 0, expected 1");
        tet_result(TET_UNRESOLVED);
        cupsFreeOptions(num_options, options);
        return;
    }
    
    errno = 0;
    cupsFreeOptions(num_options, options);
    if(errno) {
        tet_printf("cupsFreeOptions returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
}


void test17()
{
    int             org_dests = 0;    
    int             num_dests = 0;
    cups_dest_t     *dests;
    cups_dest_t     *dest;

    dests = (cups_dest_t *)0;
    dest = (cups_dest_t *)0;

    /* crashes with SIGSEGV */
    tet_result(TET_UNTESTED);
    return;

    /* free any existing dests - also part of bug 2645 issues, value returned would vary */
    cupsFreeDests(num_dests, dests);

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetDest is called to get a named destination from the list of destinations, the function returns a pointer to the destination or NULL");

    /* see bug 2645 - need to call cupsGetDests first */
    org_dests = cupsGetDests(&dests);

    /* add a destination */
    if((num_dests = cupsAddDest(dname, NULL, 1, &dests)) != org_dests + 1) {
        tet_printf("cupsAddDest returned %d, expected %d", num_dests, org_dests + 1);
        tet_result(TET_UNRESOLVED);
        cupsFreeDests(num_dests, dests);
        return;
    }
    
    /* XXX - FIXME - fails with NULL with no cupsd */
    num_dests = cupsGetDests(&dests);
    if((dest = cupsGetDest(NULL, NULL, num_dests, dests)) == NULL) {
        tet_printf("cupsGetDest returned NULL, expected '%s'", dests->name);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }

    cupsFreeDests(num_dests, dests);
}

void test18()
{
    int             org_dests = 0;
    int             num_dests = 0;
    int             tnum_dests = 0;
    cups_dest_t     *dests;

    dests = (cups_dest_t *)0;

    /* crashes with SIGSEGV */
    tet_result(TET_UNTESTED);
    return;

    /* free any existing dests - also part of bug 2645 issues, value returned would vary */
    cupsFreeDests(num_dests, dests);

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetDests is called to get the list of destinations from the default server, the function returns the number of destinations");
 
    /* see bug 2645 - need to call cupsGetDests first */
    org_dests = cupsGetDests(&dests);
    
    /* add a destination */
    if((num_dests = cupsAddDest(dname, NULL, 1, &dests)) != org_dests + 1) {
        tet_printf("cupsAddDest returned %d, expected %d", num_dests, org_dests + 1);
        tet_result(TET_UNRESOLVED);
        cupsFreeDests(num_dests, dests);
        return;
    }
    
    /* XXX - FIXME - fails with 0 with no cupsd - now with the rework for bug 2645, this returns 1 instead of 2? */
    if((tnum_dests = cupsGetDests(&dests)) != 1) {
        tet_printf("cupsGetDests returned %d, expected 1", tnum_dests);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }

    cupsFreeDests(tnum_dests, dests);   
}

void test19()
{
    int             num_dests = 0;
    cups_dest_t     *dests;
    int             tnum_dests;
    http_t          *http;
    
    dests = (cups_dest_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetDests2 is called to get the list of destinations from the default server, the function returns the number of destinations");
    
    /* add a destination */
    if((num_dests = cupsAddDest(dname, NULL, num_dests, &dests)) != 1) {
        tet_printf("cupsAddDest returned %d, expected 1", num_dests);
        tet_result(TET_UNRESOLVED);
        return;
    }
    
    /* see bug 3440 - getting 0 back on some systems, just accept >= 0 */
    http = httpConnect(cupsServer(), ippPort());
    if((tnum_dests = cupsGetDests2(http, &dests)) < 0) {
        tet_printf("cupsGetDests2 returned %d, expected >= 0", tnum_dests);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    httpClose(http);
}

void test20()
{
    http_t          *http;
    int             fd;
    http_status_t   status;
    char            filename[1024];
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetFd is called to get a file from the server, the function returns HTTP_OK if the file is successfully retrieved");

    http = httpConnect(cupsServer(), ippPort());
    fd = cupsTempFd(filename, sizeof(filename));
    if((status = cupsGetFd(http, getfile, fd)) != HTTP_OK) {
        tet_printf("cupsGetFd returned %d, expected HTTP_OK", status);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }

    close(fd);
    unlink(filename);
    httpClose(http);
}

void test21()
{
    http_t          *http;
    int             fd;
    char            filename[1024];
    http_status_t   status;
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetFile is called to get a file from the server, the function returns HTTP_OK if the file is successfully retrieved");

    http = httpConnect(cupsServer(), ippPort());
    fd = cupsTempFd(filename, sizeof(filename));
    
    if((status = cupsGetFile(http, getfile, filename)) != HTTP_OK) {
        tet_printf("cupsGetFile returned %d, expected HTTP_OK", status);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    
    close(fd);
    unlink(filename);
    httpClose(http);
}

void test22()
{
    int             num_options = 0;
    cups_option_t   *options;
    const char      *gvalue;

    options = (cups_option_t *)0;

    if((num_options = cupsAddOption(oname, ovalue, 0, &options)) != 1) {
        tet_printf("cupsAddOption returned %d, expected 1", num_options);
        tet_result(TET_UNRESOLVED);
        cupsFreeOptions(num_options, options);
        return;
    }

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetOption is called to get an option value, the function returns the option value or NULL");

    if((gvalue = cupsGetOption(oname, 1, options)) == NULL) {
        tet_printf("cupsGetOption returned NULL, expected, '%s'", ovalue);
        tet_result(TET_FAIL); 
    } else {
        if (strcmp(gvalue, ovalue) != 0) {
            tet_printf("cupsGetOption returned '%s', expected, '%s'", gvalue, ovalue);
            tet_result(TET_FAIL); 
        } else {
            tet_result(TET_PASS);
        }
    }

    cupsFreeOptions(num_options, options);
}

void test23()
{
    const char      *password;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsGetPassword is called to get a password from the user, the function returns the password");

    errno = 0;
    cupsSetPasswordCB(test_password_cb);
    if(errno) {
        tet_printf("cupsSetPasswordCB returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_UNRESOLVED);
    } else {
        password = cupsGetPassword(prompt);
        if(strcmp(password, testpw) != 0) {
            tet_printf("cupsGetPassword returned %s, expected %s", password, testpw);
            tet_result(TET_FAIL);
        } else {
            tet_result(TET_PASS);
        }
    }

    /* restore the default */
    cupsSetPasswordCB(NULL);
}

void test24()
{
    cups_lang_t     *language;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsLangDefault is called, the function returns the default language data");

    language = cupsLangDefault();
   
    /* XXX - FIXME this test was unreliable across distros trying to */
    /* test for 'C'. We just care that the function returns "something" */
    if(language->language == NULL) {
        tet_printf("cupsLangDefault returned NULL, expected a string for language->language");
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }

    cupsLangFree(language);
}

void test25()
{
    const char            *encoding;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsLangEncoding is called, the function returns the character encoding for the given language");

    encoding = cupsLangEncoding(NULL);
    
    /* XXX - FIXME - default is us-ascii - can we really count on this? */
    if((strcmp(encoding, "us-ascii")) != 0) {
        tet_printf("cupsLangEncoding returned '%s', expected 'us-ascii'", encoding);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
}

void test26()
{
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsLangFlush is called to flush all language data out of the cache, the function returns nothing");

    errno = 0;
    cupsLangFlush();
    if(errno) {
        tet_printf("cupsLangFlush returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
}

void test27()
{
    cups_lang_t     *language;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsLangFree is called, the function returns nothing");
    
    language = cupsLangDefault();
    
    errno = 0;
    cupsLangFree(language);
    if(errno) {
        tet_printf("cupsLangFree returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
}

void test28()
{
    cups_lang_t     *language;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsLangGet is called, the function returns language data for the requested language");

    language = cupsLangGet("fr");

    /* we set the language to "fr", so we should get that result now */
    /* XXX - FIXME - some systems return fr_, so only look at the first 2 chars */
    if((strncmp(language->language, "fr", 2)) != 0) {
        tet_printf("cupsLangGet returned '%s', expected 'fr'", language->language);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }

    cupsLangFree(language);
}

void test29()
{
    ipp_status_t    status;
    int             result;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsLastError is called to get the last IPP status code, the function returns the IPP status code from the last request");

    /* force an error */
    result = cupsCancelJob(printer, 0);
    result = cupsCancelJob(printer, 0);

    status = cupsLastError();
    if (status < 0) {
        tet_printf("cupsLastError returned %d, expected a non-negative value");
        tet_result(TET_FAIL);
    } else {
        tet_printf("cupsLastError returned %d", status);
        tet_result(TET_PASS);
    }
}

void test30()
{
    int                 num_options = 0;
    cups_option_t       *options;
    ppd_file_t          *ppd;
    int                 result;
    
    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsMarkOptions is called to mark command-line options in a PPD file, the function returns 1 if their are conflicting options");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    /* not expecting a conflict - test could probably be better */
    if((result = cupsMarkOptions(ppd, num_options, options)) == 1) {
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    ppdClose(ppd);
}

void test31()
{
    int                 num_options = 0;
    cups_option_t       *options;
    char                argv[16] = "foo=bar buz=baz";

    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsParseOptions is called to parse the options from a command-line argument, the function returns the number of options found");

    if((num_options = cupsParseOptions(argv, num_options, &options)) != 2) {
        tet_printf("cupsParseOptions returned %d, expected 2", num_options);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }

    cupsFreeOptions(num_options, options);
}

void test32()
{
    int                 num_options = 0;
    int                 job_id;
    int                 num_files = 2;
    http_t              *http;
    int                 result;
    const char          *files[2];
    cups_option_t       *options;

    options = (cups_option_t *)0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsPrintFiles2 is called to print one or more files to a printer or class on the default server, the function returns the Job ID");

    /* just 2 of the same file to test */
    files[0] = pfile;
    files[1] = pfile;
    
    http = httpConnect(cupsServer(), ippPort());
    job_id = cupsPrintFiles2(http, printer, num_files, files, jtitle, num_options, options);
    if (job_id == 0) {
        tet_printf("cupsPrintFiles2 returned 0, expected a value");
        tet_result(TET_FAIL);
    } else {
        tet_printf("cupsPrintFiles returned %d", job_id);
        tet_result(TET_PASS);
        result = cupsCancelJob(printer, job_id);
    }
    httpClose(http);
}

void test33()
{
    http_t              *http;
    int                 fd;
    http_status_t       status;
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsPutFd is called to put a file on the server, the function returns HTTP_CREATED if the file is stored successfully");
    
    fd = open(config, O_RDONLY);
    http = httpConnect(cupsServer(), ippPort());
    if((status = cupsPutFd(http, putfile, fd)) != HTTP_CREATED) {
        tet_printf("cupsPutFd returned %d, expected HTTP_CREATED", status);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    httpClose(http);
    close(fd);
    
    /* XXX - FIXME - uploading a conf reloads the server - wait a bit */
    sleep(WAIT_RELOAD);
}

void test34()
{
    http_t              *http;
    http_status_t       status;
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsPutFile is called to put a file on the server, the function returns HTTP_CREATED if the file is stored successfully");

    http = httpConnect(cupsServer(), ippPort());
    if((status = cupsPutFile(http, putfile, config)) != HTTP_CREATED) {
        tet_printf("cupsPutFile returned %d, expected HTTP_CREATED", status);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    httpClose(http);
    
    /* XXX - FIXME - uploading a conf reloads the server - wait a bit */
    sleep(WAIT_RELOAD);
}

/* bug 2645 - some interaction between this test (was test 36, and the old tests 35 and 40 (now 36 and 40)
 * by doing all the "Dest" tests before doing anything with Get/SetServ, things seem OK, for now */

void test35()
{
    int             org_dests = 0;
    int             num_dests = 0;
    cups_dest_t     *dests;

    dests = (cups_dest_t *)0;

    /* crashes with SIGPIPE */
    tet_result(TET_UNTESTED);
    return;

    /* free any existing dests - also part of bug 2645 issues, value returned would vary */
    cupsFreeDests(num_dests, dests);

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsSetDests is called to save the list of destinations for the default server, the destinations are saved to either /etc/cups/loptions (root) or ~/.cups/lpoptions (normal user) and the function returns nothing");

    /* see bug 2645 - need to call cupsGetDests first */
    org_dests = cupsGetDests(&dests);
    
    /* add a destination */
    if((num_dests = cupsAddDest(dname, NULL, 1, &dests)) !=  org_dests + 1) {
        tet_printf("cupsAddDest returned %d, expected %d", num_dests, org_dests + 1);
        tet_result(TET_UNRESOLVED);
        cupsFreeDests(num_dests, dests);    
        return;
    }

    errno = 0;
    cupsSetDests(num_dests, dests);
    if(errno) {
        tet_printf("cupsSetDests returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    
    cupsFreeDests(num_dests, dests);    
}

void test36()
{
    const char      *cupsserver;
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsServer is called to get the hostname/address of the default server, the function returns the server name");

    /* setup the test */        
    errno = 0;
    cupsSetServer(testhost);
    if (errno) {
        tet_printf("cupsSetServer returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_UNRESOLVED);
        return;
    }

    if((cupsserver = cupsServer()) == NULL) {
        tet_printf("cupsServer returned NULL, expected %s", testhost);
        tet_result(TET_FAIL);
    } else {
        if(strcmp(cupsserver, testhost) != 0) {
            tet_printf("cupsServer returned %s, expected %s", cupsserver, testhost);
            tet_result(TET_FAIL);
        } else {
            tet_result(TET_PASS);
        }
    }
    /* restore the default */
    cupsSetServer(NULL);
}

void test37()
{
    int             num_dests = 0;
    cups_dest_t     *dests;
    http_t          *http;
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsSetDests2 is called to save the list of destinations for the specified server, the destinations are saved to either /etc/cups/loptions (root) or ~/.cups/lpoptions (normal user) and the function returns 0 on success -1 on error");

    /* add a destination */
    if((num_dests = cupsAddDest(dname, NULL, num_dests, &dests)) != 1) {
        tet_printf("cupsAddDest returned %d, expected 1", num_dests);
        tet_result(TET_UNRESOLVED);
        cupsFreeDests(num_dests, dests);    
        return;
    }
    
    http = httpConnect(cupsServer(), ippPort());
    /* XXX - FIXME - FAILs if no /etc/cups/lpoptions */
    errno = 0;
    cupsSetDests2(http, num_dests, dests);
    if(errno) {
        tet_printf("cupsSetDests2 returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    httpClose(http);
    cupsFreeDests(num_dests, dests);    
}

void test38()
{
    http_encryption_t       encrypt;
    http_encryption_t       setencrypt = HTTP_ENCRYPT_REQUIRED;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsSetEncryption is called to set the encrytion preferences, the function returns nothing");

    /* set a value, then read it */
    errno = 0;
    cupsSetEncryption(setencrypt);
    if(errno) {
        tet_printf("cupsSetEncryption returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        if((encrypt = cupsEncryption()) != setencrypt) {
            tet_printf("cupsEncryption returned %d, expected %d", encrypt, setencrypt);
            tet_result(TET_FAIL);
        } else {
            tet_result(TET_PASS);
        }
    }
}

void test39()
{
    const char      *password;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsSetPasswordCB is called to set the password callback for CUPS, the function returns nothing");

    errno = 0;
    cupsSetPasswordCB(test_password_cb);
    if(errno) {
        tet_printf("cupsSetPasswordCB returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        password = cupsGetPassword(prompt);
        if(strcmp(password, testpw) != 0) {
            tet_printf("cupsGetPassword returned %s, expected %s", password, testpw);
            tet_result(TET_FAIL);
        } else {
            tet_result(TET_PASS);
        }
    }

    /* restore the default */
    cupsSetPasswordCB(NULL);
}

void test40()
{
    const char      *setserver;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsSetServer is called to set the default server name, the function returns nothing");

    errno = 0;
    cupsSetServer(testhost);
        if(errno) {
        tet_printf("cupsSetServer returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        if((setserver = cupsServer()) == NULL) {
            tet_printf("cupsServer returned NULL, expected %s", testhost);
            tet_result(TET_FAIL);
        } else {
            if(strcmp(setserver, testhost) != 0) {
                tet_printf("cupsServer returned %s, expected %s", setserver, testhost);
                tet_result(TET_FAIL);
            } else {
                tet_result(TET_PASS);
            }
        }
    }

    /* restore the default name */
    cupsSetServer(NULL);
}

void test41()
{
    const char    *setuser;
    char          *user;

    user = getlogin();

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsSetUser is called to set the default user name, the function returns nothing");

    errno = 0;
    cupsSetUser(testuser);
    if(errno) {
        tet_printf("cupsSetUser returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        if((setuser = cupsUser()) == NULL) {
            tet_printf("cupsUser returned NULL, expected %s", testuser);
            tet_result(TET_FAIL);
        } else {
            if((strcmp(setuser, testuser)) != 0) {
                tet_printf("cupsUser returned %s, expected %s", setuser, testuser);
                tet_result(TET_FAIL);
            } else {
                tet_result(TET_PASS);
            }
        }
    }

    /* restore the current username for test 43 */
    cupsSetUser(user);    
}

void test42()
{
    int             fd;
    char            filename[1024];

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsTempFd is called to create a temporary file, the function returns the file descriptor or -1 on error");

    fd = cupsTempFd(filename, sizeof(filename));
    if(fd == -1) {
        tet_printf("cupsTempFd returned -1, expected a file descriptor");
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }    
}

void test43()
{
    const char    *setuser;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When cupsUser is called to get the current user's name, the function returns the user name");

    errno = 0;
    /* bug 2736 - just set the user, rather than rely on what the system 
     * might return should have done it this way in the first place */
    cupsSetUser(testuser);
    if(errno) {
        tet_printf("cupsSetUser returned error '%s', expected nothing", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        if((setuser = cupsUser()) == NULL) {
            tet_printf("cupsUser returned NULL, expected %s", testuser);
            tet_result(TET_FAIL);
        } else {
            if((strcmp(setuser, testuser)) != 0) {
                tet_printf("cupsUser returned %s, expected %s", setuser, testuser);
                tet_result(TET_FAIL);
            } else {
                tet_result(TET_PASS);
            }
        }
    }
}
