/* cupsPPD.c test CUPS PPD ABI interfaces */
/* 9/24/2007 Stew Benedict stewb@linux-foundation.org */

/* interfaces:
ppdClose
ppdCollect
ppdConflicts
ppdEmit
ppdEmitFd
ppdEmitJCL
ppdErrorString
ppdFindAttr
ppdFindChoice
ppdFindMarkedChoice
ppdFindNextAttr
ppdFindOption
ppdIsMarked
ppdLastError
ppdMarkDefaults
ppdMarkOption
ppdOpen
ppdOpenFd
ppdOpenFile
ppdPageLength
ppdPageSize
ppdPageWidth
ppdSetConformance
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <tet_api.h>
#include <cups/ppd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

const char ppdfile[9] = "test.ppd";
const char ppdbfile[11] = "notest.ppd";
const char emitfile[9] = "emit.ppd";

void test_cleanup();
void (*tet_startup)() = NULL;
void (*tet_cleanup)() = test_cleanup;
void test1();
void test2();
void test3();
void test4();
void test5();
void test6();
void test7();
void test8();
void test9();
void test10();
void test11();
void test12();
void test13();
void test14();
void test15();
void test16();
void test17();
void test18();
void test19();
void test20();
void test21();
void test22();
void test23();

struct tet_testlist tet_testlist[] = 
{ 
    {test1,1},
    {test2,2},
    {test3,3},
    {test4,4},
    {test5,5},
    {test6,6},
    {test7,7},
    {test8,8},
    {test9,9},
    {test10,10},
    {test11,11},
    {test12,12},
    {test13,13},
    {test14,14},
    {test15,15},
    {test16,16},
    {test17,17},
    {test18,18},
    {test19,19},
    {test20,20},
    {test21,21},
    {test22,22},
    {test23,23},
    {NULL,0} 
};

void test_cleanup()
{
    unlink(emitfile);
}

void test1()
{
    ppd_file_t      *ppd;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdClose is called to free memory used by a PPD file, the function returns nothing");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    errno = 0;
    ppdClose(ppd);
    if (errno) {
        tet_printf("ppdClose returned %s", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
}

void test2()
{
    ppd_file_t      *ppd;
    ppd_choice_t    *choice;
    ppd_choice_t    **collect;
    ppd_option_t    *option;
    int             noptions;
    int             expected = 4;
    char            oname[10] = "InputSlot";
    char            cname[7] = "Manual";

    collect = calloc(sizeof(ppd_choice_t *), 1);

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdCollect is called on a section of a PPD file data, the function returns the number of options marked");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    errno = 0;
    ppdMarkDefaults(ppd);
    if (errno != 0) {
        tet_printf("ppdMarkDefaults returned %s", strerror(errno));
        tet_result(TET_UNRESOLVED);
    } else {
        if ((option = ppdFindOption(ppd, oname)) == NULL) {
            tet_printf("ppdFindOption returned NULL, expected pointer to '%s'", oname);
            tet_result(TET_UNRESOLVED);
        } else {
            if ((choice = ppdFindChoice(option, cname)) == NULL) {
                tet_printf("ppdFindChoice returned NULL, expected pointer to '%s'", cname);
                tet_result(TET_UNRESOLVED);
            } else {
                collect[0] = choice;
                if ((noptions = ppdCollect(ppd, PPD_ORDER_ANY, &collect)) != expected) {
                    tet_printf("ppdCollect returned %d, expected %d", noptions, expected);
                    tet_result(TET_FAIL);
                } else {
                    tet_result(TET_PASS);
                }
            }
        }
    }
    ppdClose(ppd);
}

void test3()
{
    ppd_file_t          *ppd;
    int                 ccount;
    
    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdConflicts is called on PPD file data, the function returns the number of conflicts");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((ccount = ppdConflicts(ppd)) == 0) {
        tet_result(TET_PASS);
    } else {
        tet_printf("ppdConflicts returned %d, expected 0", ccount);
        tet_result(TET_FAIL);
    }
    ppdClose(ppd);
}

void test4()
{
    ppd_file_t          *ppd;
    FILE                *fp;
    int                 emit;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdEmit is called the function writes code for marked options from the requested section of PPD data to a file, and returns 0 on success");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }
    if ((fp = fopen(emitfile, "w")) == NULL) {
        tet_printf("Cannot open %s for writing", emitfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((emit = ppdEmit(ppd, fp, PPD_ORDER_ANY)) == -1) {
        tet_printf("ppdEmit returned -1, expected 0");
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    ppdClose(ppd);
    fclose(fp);   
}

void test5()
{
    ppd_file_t          *ppd;
    int                 fd;
    int                 emit;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdEmitFd is called the function writes code for marked options from the requested section of PPD data to a file, and returns 0 on success");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }
    if ((fd = open(emitfile, O_WRONLY)) == -1) {
        tet_printf("Cannot open %s for writing", emitfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((emit = ppdEmitFd(ppd, fd, PPD_ORDER_ANY)) == -1) {
        tet_printf("ppdEmitFd returned -1, expected 0");
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    ppdClose(ppd);
    close(fd);   
}

void test6()
{
    ppd_file_t          *ppd;
    FILE                *fp;
    int                 emit;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdEmitJCL is called the function writes code for JCL options from the requested section of PPD data to a file, and returns 0 on success");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }
    if ((fp = fopen(emitfile, "w")) == NULL) {
        tet_printf("Cannot open %s for writing", emitfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((emit = ppdEmitJCL(ppd, fp, 1, "lsbuser", "ppdEmitJCL")) == -1) {
        tet_printf("ppdEmitJCL returned -1, expected 0");
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    ppdClose(ppd);
    fclose(fp);   
}

void test7()
{
    ppd_file_t          *ppd;
    const char          *estring;
    char                expected[28] = "Illegal main keyword string";

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdErrorString is called the function returns the text associated with a PPD status");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    estring = ppdErrorString(PPD_ILLEGAL_MAIN_KEYWORD);
    if (strcmp(estring, expected) != 0){
        tet_printf("ppdErrorString returned '%s', expected '%s'", estring, expected);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    ppdClose(ppd); 
}

void test8()
{
    ppd_file_t      *ppd;
    ppd_attr_t      *attr;
    char            attr_name[13] = "ttrasterizer";
    char            attr_str[7] = "Type42";

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdFindAttr is called the function returns the requested attribute from PPD data or NULL if not found");
    
    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((attr = ppdFindAttr(ppd, attr_name, NULL)) == NULL) {
        tet_printf("ppdFindAttr unexpectedly returned NULL");
        tet_result(TET_FAIL);
    } else {
        if (strcmp(attr->value, attr_str) != 0) {
            tet_printf("attr->value = %s, expected %s", attr->value, attr_str);
            tet_result(TET_FAIL);
        } else {
            tet_result(TET_PASS);
        }
    }
    ppdClose(ppd);
}

void test9()
{
    ppd_file_t      *ppd;
    ppd_option_t    *option;
    ppd_choice_t    *choice;
    char            oname[10] = "InputSlot";
    char            cname[7] = "Manual";

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdFindChoice is called the function returns a pointer to an option choice or NULL if not found");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((option = ppdFindOption(ppd, oname)) == NULL) {
        tet_printf("ppdFindOption returned NULL, expected pointer to '%s'", oname);
        tet_result(TET_UNRESOLVED);
    } else {
        if ((choice = ppdFindChoice(option, cname)) == NULL) {
            tet_printf("ppdFindChoice returned NULL, expected pointer to '%s'", cname);
            tet_result(TET_FAIL);
        } else {
            tet_result(TET_PASS);
        }
    }
    ppdClose(ppd);
}

void test10()
{
    ppd_file_t      *ppd;
    char            oname[10] = "InputSlot";
    ppd_choice_t    *choice;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdFindMarkedChoice is called the function returns a pointer to a marked option choice or NULL if not found");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    errno = 0;
    ppdMarkDefaults(ppd);
    if (errno != 0) {
        tet_printf("ppdMarkDefaults returned %s", strerror(errno));
        tet_result(TET_UNRESOLVED);
    } else {
        if ((choice = ppdFindMarkedChoice(ppd, oname)) == NULL) {
            tet_printf("ppdFindMarkedChoice returned NULL, expected a pointer to the marked choice of '%s'", oname);
            tet_result(TET_FAIL);
        } else {
            tet_result(TET_PASS);
        }
    }
    ppdClose(ppd);
}

void test11()
{
    ppd_file_t      *ppd;
    ppd_attr_t      *attr;
    char            aname[13] = "TTRasterizer";

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdFindNextAttr is called the function returns the next matching requested attribute or NULL if not found");
    
    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((attr = ppdFindAttr(ppd, aname, NULL)) == NULL) {
        tet_printf("ppdFindAttr unexpectedly returned NULL for '%s'", aname);
        tet_result(TET_UNRESOLVED);
    } else {
        if ((attr = ppdFindNextAttr(ppd, aname, NULL)) != NULL) {
            tet_printf("ppdFindNextAttr for '%s' unexpectedly returned a value when there is only 1 '%s' entry in '%s'", aname, aname, ppdfile);
            tet_result(TET_FAIL);
        } else {           
            tet_result(TET_PASS);
        }
    }
    ppdClose(ppd);
}

void test12()
{
    ppd_file_t      *ppd;
    char            oname[10] = "InputSlot";
    ppd_option_t    *option;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdFindOption is called the function returns a pointer to the specified option in PPD file data or NULL if not found");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((option = ppdFindOption(ppd, oname)) == NULL) {
        tet_printf("ppdFindOption returned NULL, expected a pointer to '%s'", oname);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    ppdClose(ppd);
}

void test13()
{
    ppd_file_t      *ppd;
    char            option[10] = "InputSlot";
    char            choice[7] = "Manual";
    int             result;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdIsMarked is called the function returns non-zero if the specified PPD data option is marked");
    
    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((result = ppdMarkOption(ppd, option, choice)) != 0) {
        tet_printf("ppdMarkOption returned %d", result);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if((result = ppdIsMarked(ppd, option, choice)) == 0) {
        tet_printf("ppdIsMarked returned %d, expected non-zero", result);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    ppdClose(ppd);
}

void test14()
{
    ppd_file_t      *ppd;
    int             line = 10;
    ppd_status_t    error;
    int             fails = 0;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdLastError is called the function returns the status code from the last ppdOpen call");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if((error = ppdLastError(&line)) != PPD_OK) {
        tet_printf("ppdLastError returned %d, expected PPD_OK(0)", error);
        fails++;
    }
    ppdClose(ppd);

    /* do another pass on a known bad file (empty) */
    ppd = ppdOpenFile(ppdbfile);

    if((error = ppdLastError(&line)) != PPD_MISSING_PPDADOBE4) {
        tet_printf("ppdLastError returned %d, expected PPD_MISSING_PPDADOBE4(4)", error);
        fails++;
    }
    ppdClose(ppd);

    if (fails == 0) {
        tet_result(TET_PASS);
    } else {
        tet_result(TET_FAIL);
    }
}

void test15()
{
    ppd_file_t      *ppd;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdMarkDefaults is called the function marks all the default options in PPD file data and returns nothing");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    errno = 0;
    ppdMarkDefaults(ppd);
    if (errno != 0) {
        tet_printf("ppdMarkDefaults returned %s", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    ppdClose(ppd);
}

void test16()
{
    ppd_file_t      *ppd;
    char            option[10] = "InputSlot";
    char            choice[7] = "Manual";
    int             result;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdMarkOption is called the function marks a requested option in PPD file data and returns -1 if the choice conflicts with the currently selection option or the number of conflicts");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }

    if ((result = ppdMarkOption(ppd, option, choice)) == -1) {
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
    ppdClose(ppd);
}

void test17()
{
    FILE            *fp;
    ppd_file_t      *ppd;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdOpen is called the function reads a PPD file into memory and returns the PPD file record");

    if ((fp = fopen(ppdfile, "r")) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }
    if ((ppd = ppdOpen(fp)) == NULL) {
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
        ppdClose(ppd);
    }
    fclose(fp);
}

void test18()
{
    ppd_file_t      *ppd;
    int               fd;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdOpenFd is called the function reads a PPD file into memory and returns the PPD file record");

    if ((fd = open(ppdfile, O_RDONLY)) == -1) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
        return;
    }
    if ((ppd = ppdOpenFd(fd)) == NULL) {
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
        ppdClose(ppd);
    }
    close(fd);
}

void test19()
{
    ppd_file_t      *ppd;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdOpenFile is called the function reads a PPD file into memory and returns the PPD file record");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
        ppdClose(ppd);
    }
}

void test20()
{
    ppd_file_t      *ppd;
    float           length;
    float           expected = 792;     /* Letter size length in points */

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdPageLength is called the function returns the length in points for a given page size string or 0.0");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
    } else {
        if ((length = ppdPageLength(ppd, "Letter")) != expected) {
            tet_printf("ppdPageLength returned %3.2f, expecting %3.2f", length, expected); 
            tet_result(TET_FAIL);
        } else {
            tet_result(TET_PASS);
            ppdClose(ppd);
        }
    }
}

void test21()
{
    ppd_file_t      *ppd;
    ppd_size_t      *size;
    float expected = 684.00;

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdPageSize is called the function returns the size record for a given page size string or NULL if not found");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
    } else {
        if ((size = ppdPageSize(ppd, "Env10")) == NULL) {
            tet_printf("ppdPageSize returned NULL"); 
            tet_result(TET_FAIL);
        } else {
            /* just a sanity check */
            tet_printf("size->length = %3.2f, expecting %3.2f", size->length, expected);
            tet_result(TET_PASS);
            ppdClose(ppd);
        }
    }
}

void test22()
{
    ppd_file_t      *ppd;
    float           width;
    float           expected = 612;     /* Letter size width in points */

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdPageWidth is called the function returns the width in points for a given page size string or 0.0");

    if ((ppd = ppdOpenFile(ppdfile)) == NULL) {
        tet_printf("Cannot open %s", ppdfile);
        tet_result(TET_UNRESOLVED);
    } else {
        if ((width = ppdPageWidth(ppd, "Letter")) != expected) {
            tet_printf("ppdPageWidth returned %3.2f, expecting %3.2f", width, expected); 
            tet_result(TET_FAIL);
        } else {
            tet_result(TET_PASS);
            ppdClose(ppd);
        }
    }
}

void test23()
{

    tet_printf("Test %d", tet_thistest);
    tet_infoline("When ppdSetConformance is called the function sets the conformace level for PPD files and returns nothing");

    errno = 0;
    ppdSetConformance(PPD_CONFORM_STRICT);
    if (errno) {
        tet_printf("ppdSetConformance returned %s", strerror(errno));
        tet_result(TET_FAIL);
    } else {
        tet_result(TET_PASS);
    }
}

